/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <stdlib.h>
#include <stdbool.h>
#include <pthread.h>
#include "message_port_common.h"
#include "message_port_local.h"
#include "message_port_remote.h"

#include "message_port_log.h"
#include "message_port.h"

EXPORT_API int message_port_register_local_port(const char *local_port, message_port_message_cb callback, void *user_data)
{
	int local_port_id;

	if (local_port == NULL || callback == NULL) {
		_LOGE("NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}

	message_port_lock_mutex();
	local_port_id = register_message_port(local_port, false, callback, user_data);
	message_port_unlock_mutex();

	if (local_port_id > 0)
		_LOGD("Register local port ID (%d).", local_port_id);
	else
		_LOGE("Register local port fail (%d).", local_port_id);

	return local_port_id;
}

EXPORT_API int message_port_register_trusted_local_port(const char *local_port, message_port_trusted_message_cb callback, void *user_data)
{
	int trusted_local_port_id;

	if (local_port == NULL || callback == NULL) {
		_LOGE("NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}

	message_port_lock_mutex();
	trusted_local_port_id = register_message_port(local_port, true, callback, user_data);
	message_port_unlock_mutex();

	if (trusted_local_port_id > 0)
		_LOGD("Register trusted local port ID (%d).", trusted_local_port_id);
	else
		_LOGE("Register trusted local port fail (%d).", trusted_local_port_id);

	return trusted_local_port_id;
}

EXPORT_API int message_port_unregister_local_port(int local_port_id)
{
	int res;

	if (local_port_id <= 0) {
		_LOGE("Neither 0 nor negative value is allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}

	message_port_lock_mutex();
	res = unregister_local_port(local_port_id, false);
	message_port_unlock_mutex();

	return res;
}

EXPORT_API int message_port_unregister_trusted_local_port(int trusted_local_port_id)
{
	int res;

	if (trusted_local_port_id <= 0) {
		_LOGE("Neither 0 nor negative value is allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}

	message_port_lock_mutex();
	res = unregister_local_port(trusted_local_port_id, true);
	message_port_unlock_mutex();

	return res;
}

EXPORT_API int message_port_check_remote_port(const char *remote_app_id, const char *remote_port, bool *exist)
{
	int ret;

	if (remote_app_id == NULL || remote_port == NULL) {
		_LOGE("NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}
	_LOGD("Check remote port (%s):(%s).", remote_app_id, remote_port);

	message_port_lock_mutex();
	ret = check_remote_port(remote_app_id, remote_port, false, exist);
	message_port_unlock_mutex();

	return ret;
}

EXPORT_API int message_port_check_trusted_remote_port(const char *remote_app_id, const char *remote_port, bool *exist)
{
	int ret;

	if (remote_app_id == NULL || remote_port == NULL) {
		_LOGE("NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}
	_LOGD("Check trusted remote port (%s):(%s).", remote_app_id, remote_port);
	message_port_lock_mutex();
	ret = check_remote_port(remote_app_id, remote_port, true, exist);
	message_port_unlock_mutex();

	return ret;
}

EXPORT_API int message_port_send_message(const char *remote_app_id, const char *remote_port, bundle *message)
{
	int ret;

	if (remote_app_id == NULL || remote_port == NULL || message == NULL) {
		_LOGE("NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}
	_SECURE_LOGI("Send a message to (%s):(%s).", remote_app_id, remote_port);

	message_port_lock_mutex();
	ret = send_message(remote_app_id, remote_port, NULL, false, false, false, message);
	message_port_unlock_mutex();

	return ret;
}

EXPORT_API int message_port_send_trusted_message(const char *remote_app_id, const char *remote_port, bundle *message)
{
	int ret;

	if (remote_app_id == NULL || remote_port == NULL || message == NULL) {
		_LOGE("[MESSAGE_PORT_ERROR_INVALID_PARAMETER] NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}
	_SECURE_LOGI("Send a trusted message to (%s):(%s).", remote_app_id, remote_port);

	message_port_lock_mutex();
	ret = send_message(remote_app_id, remote_port, NULL, true, false, false, message);
	message_port_unlock_mutex();

	return ret;
}

EXPORT_API int message_port_send_message_with_local_port(const char *remote_app_id, const char *remote_port, bundle *message, int local_port_id)
{
	int ret;

	if (remote_app_id == NULL || remote_port == NULL || message == NULL) {
		_LOGE("[MESSAGE_PORT_ERROR_INVALID_PARAMETER] NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	} else if (local_port_id <= 0) {
		_LOGE("[MESSAGEPORT_ERROR_INVALID_PARAMETER] Neither 0 nor negative value is allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}
	_SECURE_LOGI("Send a message to (%s):(%s) - (%d).", remote_app_id, remote_port, local_port_id);

	message_port_lock_mutex();
	ret = send_bidirectional_message(local_port_id, remote_app_id, remote_port, false, message);
	message_port_unlock_mutex();

	return ret;
}

EXPORT_API int message_port_send_trusted_message_with_local_port(const char *remote_app_id, const char *remote_port, bundle *message, int local_port_id)
{
	int ret;

	if (remote_app_id == NULL || remote_port == NULL || message == NULL) {
		_LOGE("[MESSAGE_PORT_ERROR_INVALID_PARAMETER] NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	} else if (local_port_id <= 0) {
		_LOGE("[MESSAGEPORT_ERROR_INVALID_PARAMETER] Neither 0 nor negative value is allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}
	_SECURE_LOGI("Send a trusted message to (%s):(%s) - (%d).", remote_app_id, remote_port, local_port_id);

	message_port_lock_mutex();
	ret = send_bidirectional_message(local_port_id, remote_app_id, remote_port, true, message);
	message_port_unlock_mutex();

	return ret;
}

EXPORT_API int message_port_add_registered_cb(const char *remote_app_id, const char *remote_port, bool is_trusted, message_port_registration_event_cb registered_cb, void *user_data, int *watcher_id)
{
	int ret;

	if (watcher_id == NULL || remote_app_id == NULL || remote_port == NULL ||
			registered_cb == NULL) {
		_LOGE("[MESSAGE_PORT_ERROR_INVALID_PARAMETER] NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}

	ret = watch_remote_port(watcher_id, remote_app_id, remote_port, is_trusted, registered_cb, NULL, user_data);
	if (ret != MESSAGE_PORT_ERROR_NONE)
		_SECURE_LOGI("add registered callback fail (%d).", ret);


	return ret;
}

EXPORT_API int message_port_add_unregistered_cb(const char *remote_app_id, const char *remote_port, bool is_trusted, message_port_registration_event_cb unregistered_cb, void *user_data, int *watcher_id)
{
	int ret;

	if (watcher_id == NULL || remote_app_id == NULL || remote_port == NULL ||
			unregistered_cb == NULL) {
		_LOGE("[MESSAGE_PORT_ERROR_INVALID_PARAMETER] NULL value is not allowed.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}

	ret = watch_remote_port(watcher_id, remote_app_id, remote_port, is_trusted, NULL, unregistered_cb, user_data);
	if (ret != MESSAGE_PORT_ERROR_NONE)
		_SECURE_LOGI("add registered callback fail (%d).", ret);


	return ret;
}

EXPORT_API int message_port_remove_registration_event_cb(int watcher_id)
{
	int ret = MESSAGE_PORT_ERROR_NONE;
	if (watcher_id < 1) {
		_LOGE("[MESSAGE_PORT_ERROR_INVALID_PARAMETER] Invalid watcher_id.");
		return MESSAGE_PORT_ERROR_INVALID_PARAMETER;
	}

	ret = remove_registration_event_cb(watcher_id);
	return ret;
}
